/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/

#include <iostream>
#include <GL/glut.h>
#include "deltaplane.h"

DeltaPlane::DeltaPlane(const Point& pos, double width, double span, int resolution_, double mass, double massperson)
: resolution(resolution_), controlSticks(NULL), simulator(NULL)
{
   nbPartsToile = (resolution+2) * (resolution+1) / 2;
   partsToile = new Particule[nbPartsToile];
   normalsToile = new Vector[nbPartsToile];
   for(int i=0, n=0;i<=resolution;i++){
      for(int j=0;j<=i;j++){
         double x = (2.0*j + resolution-i) / (resolution * 2) - 0.5;
         double y = (double)i/resolution - 0.5;
         double z = (-y) / 5;

         partsToile[n].pos = Point(pos.x + x*width, pos.y + y*span, pos.z + z*span);
         partsToile[n].mass = mass / nbPartsToile;
         n++;
      }
   }

   hautToile.pos = pos + Vector(0.0, 0.0, 0.2) * width;
   hautToile.mass = mass / nbPartsToile;
   hautToile.isVirtual = true;

   basToile.pos =  pos + Vector(0.0, 0.0, -0.2) * width;
   basToile.mass = mass / nbPartsToile;
   basToile.isVirtual = true;

   centre.pos = pos;
   centre.mass = mass / nbPartsToile;

   // Control
   controlCible = pos - Vector(0.0, -0.2, 0.4);
   person.pos = controlCible;
   person.mass = massperson;
   controlPoints = new Point[resolution*2+1];
   controlPoints[0] = partsToile[0].pos;
   for(int i=1, n1=1, n2=2;i<=resolution;i++){
      controlPoints[i*2-1] = partsToile[n1].pos;
      controlPoints[i*2] = partsToile[n2].pos;
      n1 += i + 1;
      n2 += i + 2;
   }
}

DeltaPlane::~DeltaPlane(){
   delete[] partsToile;
   delete[] normalsToile;
   delete[] controlPoints;
   delete[] controlSticks;
}

void DeltaPlane::registerInSimulator(PhysicSimulatorVerlet* sim){
   simulator = sim;
   for(int i=0;i<nbPartsToile;i++)
      sim->addParticule(partsToile + i);
   sim->addParticule(&hautToile);
   sim->addParticule(&centre);
   sim->addParticule(&basToile);
   sim->addParticule(&person);

   for(int i=0, n=0;i<resolution;i++){
      for(int j=0;j<=i;j++){
         Particule* a = partsToile + n;
         Particule* b = partsToile + n + (i+1);
         Particule* c = partsToile + n + (i+2);

         sim->addStick(Stick(a, b));
         sim->addStick(Stick(a, c));
         sim->addStick(Stick(b, c));

         sim->addSurface(SurfaceTriangle(a, b, c));
         if(j<i){
            Particule* d = partsToile + n + 1;
            sim->addSurface(SurfaceTriangle(a, c, d));
         }
         n++;
      }
   }

   // Rigidit des cts
   sim->addStick(Stick(&hautToile, &basToile));
   sim->addStick(Stick(&hautToile, &centre));
   sim->addStick(Stick(&centre, &basToile));
   controlSticks = new int[resolution*2+1];
   for(int i=0, n1=0, n2=0,cs=0;i<=resolution;i++){
       sim->addStick(Stick(partsToile + n1, &hautToile));
       sim->addStick(Stick(partsToile + n1, &centre));
       sim->addStick(Stick(partsToile + n1, &basToile));
       controlSticks[cs++] = sim->addStick(Stick(partsToile + n1, &person));
       if(n2!=n1){
          controlSticks[cs++] = sim->addStick(Stick(partsToile + n2, &person));
          sim->addStick(Stick(partsToile + n1, partsToile + n2));
          sim->addStick(Stick(partsToile + n2, &hautToile));
          sim->addStick(Stick(partsToile + n2, &centre));
          sim->addStick(Stick(partsToile + n2, &basToile));
       }
       n1 += i+1;
       n2 += i+2;
   }

   sim->addStick(Stick(&centre, &person));

   // Rigidit du centre
   for(int i=0,n=0;i<=resolution/2;i++){
      sim->addStick(Stick(partsToile + n, &hautToile));
      sim->addStick(Stick(partsToile + n, &basToile));
      n += (i+1) * 4;
   }

   //Rigidit du bout
   for(int i=0;i<=resolution;i++){
       //Particule* p = partsToile + nbPartsToile - (resolution - i + 1);
       //int j = i <= resolution/2 ? i : i - resolution/2;
       //j-= resolution/4;
       //p->pos.z += j*j / 10.0;
       //Stick s = Stick(p, &hautToile);
       //sim->addStick(s);
       //s = Stick(p, &basToile);
       //sim->addStick(s);
   }

}

void DeltaPlane::computeNormals(){

   for(int i=0;i<nbPartsToile;i++)
      normalsToile[i].setZero();

/*
 //Ancienne faon d'afficher : le centre n'est pas bien fixe....
   for(int i=0,n=0;i<resolution;i++){
      for(int j=0;j<=i;j++){
         int a = n;
         int b = n + i + 1;
         int c = n + i + 2;

	 Vector v1 = partsToile[b].pos - partsToile[a].pos;
	 Vector v2 = partsToile[c].pos - partsToile[b].pos;
	 Vector vn = v2 ^ v1;
	 vn.normalize();
	 normalsToile[a] += vn;
	 normalsToile[b] += vn;
	 normalsToile[c] += vn;

	 if(i<i){
	    int d = n + 2;
	    v1 = partsToile[d].pos - partsToile[a].pos;
	    v2 = partsToile[c].pos - partsToile[a].pos;
	    vn = v2 ^ v1;
	    vn.normalize();
            normalsToile[a] += vn;
	    normalsToile[c] += vn;
	    normalsToile[d] += vn;
	 }

         n++;
      }
   }
*/

   //
   for(int i=0, a=0;i<resolution;i++){
     for(int j=0;j<=i;j++){
        int b = a + i + 1;
        int c = a + i + 2;
        if(i==resolution-1){
            Vector v1 = partsToile[b].pos - partsToile[a].pos;
            Vector v2 = partsToile[c].pos - partsToile[b].pos;
            Vector vn = v1 ^ v2;
            vn.normalize();
            normalsToile[a] += vn;
            normalsToile[b] += vn;
            normalsToile[c] += vn;
        }else{
           int d = c + i + 2;
            Vector v1 = partsToile[b].pos - partsToile[a].pos;
            Vector v2 = partsToile[d].pos - partsToile[b].pos;
            Vector vn = v1 ^ v2;
            vn.normalize();
            normalsToile[a] += vn;
            normalsToile[b] += vn;
            normalsToile[d] += vn;

            v1 = partsToile[d].pos - partsToile[a].pos;
            v2 = partsToile[c].pos - partsToile[d].pos;
            vn = v1 ^ v2;
            vn.normalize();
            normalsToile[a] += vn;
            normalsToile[c] += vn;
            normalsToile[d] += vn;
        }
        a++;
     }
   }


   for(int i=0;i<nbPartsToile;i++){
      normalsToile[i].normalize();
      //normalsToile[i] *= -1;
   }

}

void DeltaPlane::renderGL() const {
   ((DeltaPlane*)this)->computeNormals();
   glPushMatrix();
   const Point* p;
   const Vector* v;

   glColor3f(0.8f, 0.2f, 0.1f);

/*
   for(int i=0,n=0;i<resolution;i++){
      glBegin(GL_TRIANGLE_STRIP);
      for(int j=0;j<=i;j++){
         p = &partsToile[n + i + 1].pos;
	 v = normalsToile + n + i + 1;
	 glNormal3d(v->x, v->y, v->z);
         glVertex3d(p->x, p->y, p->z);

         p = &partsToile[n].pos;
	 v = normalsToile + n;
	 glNormal3d(v->x, v->y, v->z);
         glVertex3d(p->x, p->y, p->z);

         n++;
      }
      p = &partsToile[n + i + 1].pos;
      v = normalsToile + n + i + 1;
      glNormal3d(v->x, v->y, v->z);
      glVertex3d(p->x, p->y, p->z);
      glEnd();
   }
*/
   //
   glBegin(GL_TRIANGLES);
   for(int i=0, a=0;i<resolution;i++){
     for(int j=0;j<=i;j++){
        int b = a + i + 1;
        int c = a + i + 2;
        if(i==resolution-1){
           v = normalsToile + a;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[a].pos;
            glVertex3d(p->x, p->y, p->z);
           v = normalsToile + b;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[b].pos;
            glVertex3d(p->x, p->y, p->z);
           v = normalsToile + c;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[c].pos;
            glVertex3d(p->x, p->y, p->z);
        }else{
           int d = c + i + 2;
           v = normalsToile + a;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[a].pos;
            glVertex3d(p->x, p->y, p->z);
           v = normalsToile + b;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[b].pos;
            glVertex3d(p->x, p->y, p->z);
           v = normalsToile + d;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[d].pos;
            glVertex3d(p->x, p->y, p->z);

           v = normalsToile + a;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[a].pos;
            glVertex3d(p->x, p->y, p->z);
           v = normalsToile + d;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[d].pos;
            glVertex3d(p->x, p->y, p->z);
           v = normalsToile + c;
            glNormal3d(v->x, v->y, v->z);
            p = &partsToile[c].pos;
            glVertex3d(p->x, p->y, p->z);
        }
        a++;
     }
   }
   glEnd();

   // Display person
   glColor3f(0.2f, 0.3f, 0.8f);
   glBegin(GL_LINES);
     p = &person.pos;
     glVertex3f(p->x, p->y, p->z);
     p = &centre.pos;
     glVertex3f(p->x, p->y, p->z);
   glEnd();
   glPushMatrix();
     p = &person.pos;
     glTranslated(p->x, p->y, p->z);
     glutSolidSphere(0.3, 8, 8);
   glPopMatrix();




   // Rigidit des cts
/*   glColor3f(0.2, 0.2, 0.7);
   glBegin(GL_LINES);
   for(int i=0, n1=0, n2=0;i<=resolution;i++){
       p = &partsToile[n1].pos;
       glVertex3d(p->x, p->y, p->z);

       p = &hautToile.pos;
       glVertex3d(p->x, p->y, p->z);

       if(n2!=n1){
          glVertex3d(p->x, p->y, p->z);
          p = &partsToile[n2].pos;
          glVertex3d(p->x, p->y, p->z);
       }


       p = &partsToile[n1].pos;
       glVertex3d(p->x, p->y, p->z);

       p = &basToile.pos;
       glVertex3d(p->x, p->y, p->z);

       if(n2!=n1){
          glVertex3d(p->x, p->y, p->z);
          p = &partsToile[n2].pos;
          glVertex3d(p->x, p->y, p->z);
       }

       n1 += i+1;
       n2 += i+2;
   }
   glEnd();
*/

   glPopMatrix();

}

void DeltaPlane::controlPosition(double dx, double dy){
   controlCible.x += dx;
   controlCible.y += dy;
   for(int i=0;i<resolution*2+1;i++){
      Stick& s = simulator->getStick(controlSticks[i]);
      double l = controlPoints[i].distance(controlCible);
      s.setDistance(l);
   }
}

void DeltaPlane::moveTo(const Point& dst){
  Vector v = dst - centre.pos;
  simulator->move(v);
}


