/*
   (C) Eric Beaudry 2004.
*/

#include <iostream>
#include <GL/glut.h>
#include <vector>
#include "object.h"
#include "displayglut.h"

float        distanceCamera;
float        angleSol;
float        angleRotation;
float        refX, refY, refZ;

int          mode;
int          curX, curY;

std::vector<const Object*>   objects;
PhysicSimulatorVerlet* simulatorToDisplay;


void addObject(const Object* o){
   objects.push_back(o);
}

void showPhysicSim(PhysicSimulatorVerlet* sim){
  simulatorToDisplay = sim;
}

void init(){
    GLfloat pos[4] = {-400, 400.0, 400.0, 1.0 };
    glLightfv( GL_LIGHT0, GL_POSITION, pos );
    //glEnable( GL_CULL_FACE );
    glEnable( GL_LIGHTING );
    glEnable( GL_LIGHT0 );
    glEnable( GL_NORMALIZE );
    glEnable(GL_COLOR_MATERIAL);
    glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,GL_TRUE);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_ALPHA_TEST);
    glShadeModel(GL_SMOOTH);

    distanceCamera = 10;
    angleSol = 30;
    angleRotation = -170;
    refX = refY = refZ = 0;
    mode = 0;
}

void setPos(double x, double y, double z){
  refX = x;
  refY = y;
  refZ = z;
}

void display(){
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glTranslatef(0, 0, - distanceCamera);
    glRotatef(angleSol-90, 1, 0, 0);
    glRotatef(angleRotation, 0,0,1);
    glTranslatef(-refX, -refY, -refZ);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    for(unsigned int i=0;i<objects.size();i++)
        objects[i]->renderGL();

    if(simulatorToDisplay)
       simulatorToDisplay->renderGL();

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glFlush();
    glutSwapBuffers();
}

void reshape(int width, int height){
    GLfloat w = (float) width / (float) height;
    GLfloat h = 1.0;

    glViewport( 0, 0, width, height );
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum( -w, w, -h, h, 2.0, 400.0 );
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void mousefonc(int button, int state, int x, int y){
   //GLUT_LEFT_BUTTON
   //GLUT_UP, GLUT_DOWN
   if(button==GLUT_LEFT_BUTTON && state==GLUT_DOWN){
      mode = (glutGetModifiers() & GLUT_ACTIVE_SHIFT)==0 ? 1 : 2;
      curX = x;
      curY = y;
      glutGetModifiers();
   }
   else
      mode = 0;
}

void mousemove(int x, int y){
   int dx = curX - x;
   int dy = curY - y;
   switch(mode){
      case 2:
         distanceCamera += dy / 10.0;
         if(distanceCamera<2) distanceCamera = 2;
         else if(distanceCamera>1000) distanceCamera = 1000;
         glutPostRedisplay();
         break;
      case 1:
         angleSol += dy * 0.25;
         if(angleSol < 0) angleSol = 0;
         else if(angleSol > 90) angleSol =90;
         angleRotation += -dx * 0.25;
         glutPostRedisplay();
         break;
   }
   curX = x;
   curY = y;
}

void special(int key, int x, int y){
   std::cout << "special key: " << key << std::endl;
}

void keyboard(unsigned char key, int x, int y){
   if(key==27)
     exit(0);
   std::cout << "key: " << key << std::endl;
}

int createGlut(int* argc, char** argv){
   glutInit(argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
   glutInitWindowSize(800, 600);
   glutInitWindowPosition(100,100);
   glutCreateWindow(argv[0]);
   init();
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutMouseFunc(mousefonc);
   glutMotionFunc(mousemove);
   glutKeyboardFunc(keyboard);
   glutSpecialFunc(special);
   return 0;
}
