/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/


#include <GL/glut.h>
#include <iostream>

#include "drapeau.h"
#include "displayglut.h"
#include "object.h"
#include "physic.h"


Drapeau::Drapeau(int resw, int resh, double x, double y, double z, double w, double h, double m)
 : resW(resw+1), resH(resh+1)
{
   particules = new Particule[resW*resH];
   normals = new Vector[resW*resH];
   for(int j=0,n=0;j<resH;j++)
     for(int i=0;i<resW;i++){
        particules[n].pos = Point(x + i*w/resW, y, z - j*h/resH);
        particules[n].mass = m / resW*resH;
        n++;
     }
}

Drapeau::~Drapeau(){
  delete[] particules;
  delete[] normals;
}

void Drapeau::registerInSimulator(PhysicSimulatorEuler* sim, double k, double d) {
   int n = resW * resH;
   for(int i=0;i<n;i++)
      sim->addParticule(&particules[i]);

   sim->addNail(Nail(particules + 0));
   //sim->addNail(Nail(particules + resW-1));
   //sim->addNail(Nail(particules + 1 * resW));
   //sim->addNail(Nail(particules + 2 * resW));
   sim->addNail(Nail(particules + (resH-1) * resW));

   //for(int i=0;i<resH;i++)
   //   sim->addNail(Nail(particules + resW*i));

   // Set ressors
   // Horizontal
   for(int j=0;j<resH;j++)
     for(int i=0;i<resW-1;i++)
        sim->addSpring(Spring(&get(i, j), &get(i+1, j), k, d));

   // Vertical
   for(int j=0;j<resH-1;j++)
     for(int i=0;i<resW;i++)
        sim->addSpring(Spring(&get(i, j), &get(i, j+1), k, d));

   // X
   for(int j=0;j<resH-1;j++)
     for(int i=0;i<resW-1;i++){
        sim->addSpring(Spring(&get(i, j), &get(i+1, j+1), k*1.2, d));
        sim->addSpring(Spring(&get(i+1, j), &get(i, j+1), k*1.2, d));
        sim->addSurface(SurfaceTriangle(&get(i, j), &get(i, j+1), &get(i+1, j)));
        sim->addSurface(SurfaceTriangle(&get(i+1, j+1), &get(i+1, j), &get(i, j+1)));
     }
}

void Drapeau::registerInSimulator(PhysicSimulatorVerlet* sim){
   int n = resW * resH;
   for(int i=0;i<n;i++)
      sim->addParticule(&particules[i]);

   sim->addNail(Nail(particules + 0));
   //sim->addNail(Nail(particules + resW-1));
   //sim->addNail(Nail(particules + 1 * resW));
   //sim->addNail(Nail(particules + 2 * resW));
   sim->addNail(Nail(particules + (resH-1) * resW));

   //for(int i=0;i<resH;i++)
   //   sim->addNail(Nail(particules + resW*i));

   // Set sticks
   // Horizontal
   for(int j=0;j<resH;j++)
     for(int i=0;i<resW-1;i++)
        sim->addStick(Stick(&get(i, j), &get(i+1, j)));

   // Vertical
   for(int j=0;j<resH-1;j++)
     for(int i=0;i<resW;i++)
        sim->addStick(Stick(&get(i, j), &get(i, j+1)));

   // X
   for(int j=0;j<resH-1;j++)
     for(int i=0;i<resW-1;i++){
        sim->addStick(Stick(&get(i, j), &get(i+1, j+1)));
        sim->addStick(Stick(&get(i+1, j), &get(i, j+1)));
        sim->addSurface(SurfaceTriangle(&get(i, j), &get(i, j+1), &get(i+1, j)));
        sim->addSurface(SurfaceTriangle(&get(i+1, j+1), &get(i+1, j), &get(i, j+1)));
     }
}

void Drapeau::renderGL() const{

   glColor3f(0.7f, 0.2f, 0.3f);

   for(int i=0;i<resH*resW;i++)
      normals[i].setZero();

   for(int j=0;j<resH-1;j++)
      for(int i=0;i<resW-1;i++){
        int i1 = i + j*resW;
        int i2 = i + (j+1)*resW;
        int i3 = (i+1) + j*resW;
        int i4 = (i+1) + (j+1)*resW;
        Point* p1 = &particules[i1].pos;
        Point* p2 = &particules[i2].pos;
        Point* p3 = &particules[i3].pos;
        Point* p4 = &particules[i4].pos;

        Vector vn = (*p2 - *p1) ^ (*p3 - *p1);
        vn.normalize();
        normals[i1] += vn;
        normals[i2] += vn;
        normals[i3] += vn;

        vn = (*p3 - *p4) ^ (*p1 - *p3);
        vn.normalize();
        normals[i1] += vn;
        normals[i3] += vn;
        normals[i4] += vn;

        vn = (*p4 - *p2) ^ (*p3 - *p4);
        vn.normalize();
        normals[i2] += vn;
        normals[i3] += vn;
        normals[i4] += vn;

        vn = (*p2 - *p1) ^ (*p2 - *p4);
        vn.normalize();
        normals[i1] += vn;
        normals[i2] += vn;
        normals[i4] += vn;
      }

   for(int i=0;i<resH*resW;i++)
      normals[i].normalize();

   for(int j=0;j<resH-1;j++)
      for(int i=0;i<resW-1;i++){
        Point* p1 = &particules[i + j*resW].pos;
        Point* p2 = &particules[i + (j+1)*resW].pos;
        Point* p3 = &particules[(i+1) + j*resW].pos;
        Vector* vn;


        glBegin(GL_POLYGON);
          vn = &normals[i+j*resW];
          glNormal3d(vn->x, vn->y, vn->z);
          glColor3f(0.7f * i / resW, 0.7f - 0.7f * i / resW, 0.3f);
          glVertex3d(p1->x, p1->y, p1->z);

          vn = &normals[i+(j+1)*resW];
          glNormal3d(vn->x, vn->y, vn->z);
          glColor3f(0.7f * i / resW, 0.7f - 0.7f * i / resW, 0.3f);
          glVertex3d(p2->x, p2->y, p2->z);

          vn = &normals[(i+1)+j*resW];
          glNormal3d(vn->x, vn->y, vn->z);
          glColor3f(0.7f * (i+1) / resW, 0.7f - 0.7f * (i+1) / resW, 0.3f);
          glVertex3d(p3->x, p3->y, p3->z);
        glEnd();


        p1 = &particules[(i+1) + (j+1)*resW].pos;
        glBegin(GL_POLYGON);
          vn = &normals[(i+1)+(j+1)*resW];
          glNormal3d(vn->x, vn->y, vn->z);
          glColor3f(0.7 * (i+1) / resW, 0.7f - 0.7f * (i+1) / resW, 0.3f);
          glVertex3d(p1->x, p1->y, p1->z);

          vn = &normals[(i+1)+j*resW];
          glNormal3d(vn->x, vn->y, vn->z);
          glColor3f(0.7f * (i+1) / resW, 0.7f - 0.7f * (i+1) / resW, 0.3f);
          glVertex3d(p3->x, p3->y, p3->z);

          vn = &normals[i+(j+1)*resW];
          glNormal3d(vn->x, vn->y, vn->z);
          glColor3f(0.7f * i / resW, 0.7f - 0.7f * i / resW, 0.3f);
          glVertex3d(p2->x, p2->y, p2->z);

        glEnd();

     }

}

