/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://siela.dmi.usherb.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/

#if !defined(__GEOMETRIE_H__)
#define __GEOMETRIE_H__

#include <ostream>

class Vector;

class Point {
  public:
    Point(double x=0, double y=0, double z=0);

    Point   operator +  (const Vector& v) const;
    Point&  operator += (const Vector& v);

    Point   operator -  (const Vector& v) const;
    Point&  operator -= (const Vector& v);

    Vector  operator -  (const Point& p) const;

    Point   operator *  (const double& v) const;
    Point&   operator *=  (const double& v);

    double  distance(const Point& p) const;
    double  distance2(const Point& p) const;

    double        x, y, z;

};


class Vector{
  public:
    Vector(double x=0, double y=0, double z=0);
    Vector(const Point& p1, const Point& p2);

    Vector   operator +  (const Vector& v) const;
    Vector&  operator += (const Vector& v);

    Vector   operator -  () const;
    Vector   operator -  (const Vector& v) const;
    Vector&  operator -= (const Vector& v);

    Vector   operator ^  (const Vector& v) const;
    Vector&  operator ^= (const Vector& v);

    Vector   operator *  (const double& v) const;
    Vector   operator *= (const double& v);

    Vector   operator /  (const double& v) const;
    Vector   operator /= (const double& v);

    double   operator *  (const Vector& v) const;

    double   norm() const;
    double   norm2() const;

    void     normalize();

    void     setZero(){x=y=z=0;}

    Vector   projectOn(const Vector& u) const;

    double        x, y, z;
};

std::ostream& operator << (std::ostream& out, const Point&);
std::ostream& operator << (std::ostream& out, const Vector&);

#endif
