/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

   Note: Terrain (Height Field)

*/

#include "terrain.h"
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <GL/glut.h>
extern "C"{
#include <jpeglib.h>
}
#include <jerror.h>

Terrain::Terrain(int w, int h, double ox_, double oy_, double resolution_, double niveau)
: width(w), height(h), resolution(resolution_), ox(ox_), oy(oy_)
{
   ep_eau = new double[w*h];
   nivsol = new double[w*h];
   normals = NULL;
   for(int i=0;i<w*h;i++){
      ep_eau[i] = 0;
      nivsol[i] = niveau;
   }
}

Terrain::~Terrain(){
   delete[] nivsol;
   delete[] ep_eau;
   delete[] normals;
}

void Terrain::lisser(){
   double* nouveau = new double[width*height];
   for(int x=0;x<width;x++)
    for(int y=0;y<height;y++){
       double t = 0;
       int c=0;
       for(int x2=x-1;x2<=x-1;x2++)
         for(int y2=y-1;y2<=y-1;y2++)
	    if(x2>=0 && y2>=0 && x2<width && y2<height){
	      t += sol(x2, y2);
	      c++;
	    }
       nouveau[x + y*width] = t/c;
    }
   delete[] nivsol;
   nivsol = nouveau;
}

void  Terrain::approcherVoisin(int x, int y, double cible, double maxdiff){
   double diff = sol(x, y) - cible;
   if(diff > maxdiff){
      sol(x, y) = cible + maxdiff;
   }else if(diff < -maxdiff){
      sol(x, y) = cible - maxdiff;
   }else return;

   if(x>0) approcherVoisin(x-1, y, sol(x, y), maxdiff);
   if(x<width-1) approcherVoisin(x+1, y, sol(x, y), maxdiff);
   if(y>0) approcherVoisin(x, y-1, sol(x, y), maxdiff);
   if(y<height-1) approcherVoisin(x, y+1, sol(x, y), maxdiff);

}

void  Terrain::getMinMax(double& min, double& max) const{
   int n = width * height;
   min = max = nivsol[0];
   for(int i=1;i<n;i++){
      if(nivsol[n] < min) min = nivsol[n];
      else if(nivsol[n] > max) max = nivsol[n];
   }
}

void Terrain::renderGL() const{
    float scaleVert = (1-0.3) / (maxTerrain-minTerrain);
    for(int x=0;x<width-1;x++){
      glBegin(GL_TRIANGLE_STRIP);
      for(int y=0;y<height;y++){
         double xx = ox + x * resolution;
         double yy = oy + y * resolution;
         double zz = sol(x, y);
         const Vector& vn = getVN(x, y);
         glNormal3f(vn.x, vn.y, vn.z);
         //if(displayTexture)
            //glTexCoord2d( (double)x / meshSol->getWidth(), (double)y/meshSol->getHeight());
         //else
            glColor3f(0.5, 0.3 + (zz-minTerrain) * scaleVert, 0.5);
         glVertex3f(xx, yy, zz);

         xx = ox + (x+1) * resolution;
         yy = oy + y * resolution;
         zz = sol(x+1, y);
         const Vector& vn2 = getVN(x+1, y);
         glNormal3f(vn2.x, vn2.y, vn2.z);
         //if(displayTexture)
            //glTexCoord2d( (double)x / meshSol->getWidth(), (double)y/meshSol->getHeight());
         //else
            glColor3f(0.5, 0.3 + (zz-minTerrain) * scaleVert, 0.5);
         glVertex3f(xx, yy, zz);
      }
      glEnd();
    }
}

void Terrain::computeVNs(){
   int n = width * height;
   if(normals){
      for(int i=0;i<n;i++)
         normals[i].setZero();
   }else
      normals = new Vector[width * height];

   // Vector
   for(int x=0;x<width-1;x++)
      for(int y=0;y<height-1;y++){
         Point p0(ox + x   * resolution, oy +  y  * resolution, sol(x  , y  ));
         Point p1(ox + x   * resolution, oy +(y+1)* resolution, sol(x  , y+1));
         Point p2(ox +(x+1)* resolution, oy +(y+1)* resolution, sol(x+1, y+1));
         Point p3(ox +(x+1)* resolution, oy +  y  * resolution, sol(x+1, y  ));

         Vector vn = (p1 - p0) ^ (p2 - p1);
         vn.normalize();
         getVN(x  , y  ) -= vn;
         getVN(x  , y+1) -= vn;
         getVN(x+1, y+1) -= vn;

         vn = (p3 - p2) ^ (p0 - p3);
         vn.normalize();
         getVN(x  , y  ) -= vn;
         getVN(x+1, y  ) -= vn;
         getVN(x+1, y+1) -= vn;
      }

   minTerrain = maxTerrain = nivsol[0];
   for(int i=0;i<n;i++){
      normals[i].normalize();
      double z = nivsol[i];
      if(z<minTerrain) minTerrain = z;
      else if(z>maxTerrain) maxTerrain = z;
   }
}

double Terrain::getLevel(const double x, const double y) const {
   double cx = (x - ox) / resolution;
   double cy = (y - oy) / resolution;
   if(cx<0) cx = 0;
     else if(cx>=width-1) cx = width-1;
   if(cy<0) cy = 0;
     else if(cy>=height-1) cy = height-1;

   double p1 = cx - floor(cx);
   double z;
   z = p1==0 ? sol((int)cx, (int)cy)
             : (1-p1)*sol((int)cx, (int)cy) + p1*sol((int)cx + 1, (int)cy);
   double p2 = cy - floor(cy);
   if(p2>0){
     double z2 = p1==0 ? sol((int)cx, (int)cy+1)
                       : (1-p1)*sol((int)cx, (int)cy+1) + p1*sol((int)cx + 1, (int)cy+1);
     z = p2*z + (1-p2)*z2;
   }
   return z;
}

bool Terrain::testCollision(const Point& oldpos, Point& newpos) {
   double z = getLevel(newpos.x, newpos.y);
   if(newpos.z < z){
     newpos.z = z;
     return true;
   }else
     return false;
}

Terrain* Terrain::generateSinSin(int w, int h, double amplitude){
   double res = 5;
   Terrain* terrain = new Terrain(w, h, -res*w/2, -res*h/2, res);
   for(int x=0;x<w;x++)
    for(int y=0;y<h;y++){
       float xp = x-w/2;
       float yp = y-h/2;
       float zp = 0 + amplitude * (sin(xp/4) + sin(yp/4) + (0.10 * rand() / (RAND_MAX+1)));
       terrain->sol(x,y) = zp;
    }
   terrain->computeVNs();
   return terrain;
}

Terrain* Terrain::generateRandom(int w, int h, int nbmont, int nblac){
   if(w<20) w=20;
   if(h<20) h=20;
   Terrain* terrain = new Terrain(w, h);
   for(int x=0;x<w;x++){
     terrain->approcherVoisin(x, 0, 5, 0.5);
     terrain->approcherVoisin(x, h-1, 5, 0.5);
   }
   for(int y=0;y<h;y++){
     terrain->approcherVoisin(0, y, 5, 0.5);
     terrain->approcherVoisin(w-1, y, 5, 0.5);
   }

   time_t t = time(NULL);
   srand(t);

   // Gnrer des montagnes
   for(int i=0;i<nbmont + nblac;i++){
      int x1 = (int) (  (w-20.0) * rand() / (RAND_MAX+1.0)) + 10;
      int x2 = (int) (  10.0 * rand() / (RAND_MAX+1.0)) + x1;
      int y1 = (int) (  (h-20.0) * rand() / (RAND_MAX+1.0)) + 10;
      int y2 = (int) (  10.0 * rand() / (RAND_MAX+1.0)) + y1;

      int sens = i<nbmont ? 1 : -1;
      double hauteur = 5.0 * rand() / (RAND_MAX+1.0);
      if(i>nbmont)
         hauteur *= -1;

      hauteur += terrain->sol((x1+x2)/2, (y1+y2)/2);

      for(int x=x1;x<=x2;x++)
        for(int y=y1;y<y2;y++){
	   double dx = (x - x1) * 2.0 / (x2+x1) - 1.0;
	   double dy = (x - x1) * 2.0 / (x2+x1) - 1.0;
	   double he = (1 - dx*dx + dy*dy) * sens;
	   terrain->approcherVoisin(x, y, hauteur+he, 0.25);
	   terrain->sol(x, y) = hauteur+he;
	}
   }
   terrain->computeVNs();
   return terrain;
}

Terrain* Terrain::loadFromJPEG(const char* filename, double resXY, double resZ){
    FILE *file;
    struct jpeg_decompress_struct cinfo;
    struct jpeg_error_mgr jerr;

    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_decompress(&cinfo);

    if (!(file=fopen(filename,"rb"))) {
       fprintf(stderr,"Erreur : impossible d'ouvrir %s\n",filename);
       return NULL;
    }

    jpeg_stdio_src(&cinfo, file);
    jpeg_read_header(&cinfo, TRUE);
    //if ((cinfo.image_width!=256) || (cinfo.image_height!=256)) {
    //    fprintf(stderr,"Erreur : l'image doit etre de taille 256x256\n");
    //    exit(1);
    //}

    if (cinfo.out_color_space!=JCS_GRAYSCALE){
       fprintf(stderr,"Error : l'image doit etre en niveaux de gris\n");
       return NULL;
    }
    jpeg_start_decompress(&cinfo);

    Terrain* terrain = new Terrain(cinfo.output_width, cinfo.output_height,
                                  -(cinfo.output_width*resXY/2), -(cinfo.output_height*resXY/2), // les parenthses -( ...) sont trs importantes... car les dimensions sont non signes!!!
                                  resXY, 0);

    //Terrain* terrain = new Terrain(256, 256, -128, -128, 1, 0);

    unsigned char* buffer = new unsigned char[cinfo.output_width * cinfo.output_components];

    while (cinfo.output_scanline<cinfo.output_height){
       int y = cinfo.output_scanline;
       jpeg_read_scanlines(&cinfo, &buffer, 1);
       for(unsigned int x=0;x<cinfo.output_width;x++){
          terrain->sol(x, y) = buffer[x] * resZ;
       }
    }

    delete[] buffer;
    jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);
    terrain->computeVNs();
    return terrain;
}


// ---------------
void Terrain2::renderGL() const{
    glPushMatrix();

    for(int x=-10;x<10;x++)
        for(int y=-10;y<10;y++){
            if((x+y) % 2 == 0)
                glColor3ub(22, 150, 1);
            else
                glColor3ub(32, 185, 4);
            glNormal3f(0, 0, 1);
            glBegin(GL_POLYGON);
            glVertex3d( 10*(x+1), 10*y, -0.05);
            glVertex3d( 10*(x+1), 10*(y+1), -0.05);
            glVertex3d( 10*x, 10*(y+1), -0.05);
            glVertex3d( 10*x, 10*y, -0.05);
            glEnd();
        }
    glPopMatrix();
}
