/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/

#include <iostream>
#include <GL/glut.h>

#include "drapeau.h"
#include "displayglut.h"
#include "timer.h"

class TerrainA : public Object{
  public:
    void renderGL() const{
        glPushMatrix();
        glColor3f(0.7f, 0.7f, 0.7f);
        glNormal3f(0, 0, 1);
        glBegin(GL_POLYGON);
          glVertex3f(-10, -10, -0.05f);
          glVertex3f(10, -10, -0.05f);
          glVertex3f(10, 10, -0.05f);
          glVertex3f(-10, 10, -0.05f);
        glEnd();

        GLUquadricObj *obj;
        obj = gluNewQuadric();
        glColor3f(0.7f, 0.7f, 0.9f);
        gluCylinder(obj, 0.03, 0.03, 2.5, 20, 2);

        glTranslated(2.0, 0, 0);
        gluCylinder(obj, 0.03, 0.03, 2.5, 20, 2);

        gluDeleteQuadric(obj);

        glPopMatrix();
    }
};

TerrainA*           terrain;
Drapeau*           drapeau;
PhysicSimulatorEuler*     simulatorEuler;
PhysicSimulatorVerlet*    simulator;
Timer              timer;

void idle(){
   int n = timer.getNbStepToDo();
   for(int i=0;i<n;i++){
      simulator->iter();
      for(int j=0;j<10;j++)
          simulatorEuler->iter();
   }
   if(n>0)
      glutPostRedisplay();
   //else
   //   timer.waitUnitNextStep();
}

int main(int argc, char** argv){
   timer = Timer(200, 4);
   terrain = new TerrainA();
   drapeau = new Drapeau(20, 12, 0, 0, 2.5, 1.25, 0.75, 0.0270);
   simulator = new PhysicSimulatorVerlet(9.81, 100);
   simulator->setWind(Vector(9, 0.5, 0.2));
   drapeau->registerInSimulator(simulator);

   simulatorEuler = new PhysicSimulatorEuler(9.81, 1000);
   Drapeau* drapeau2 = new Drapeau(20, 12, 2.0, 0, 2.5, 1.25, 0.75, 0.0270);
   drapeau2->registerInSimulator(simulatorEuler);
   simulatorEuler->setWind(Vector(9, 0.5, 0.2));
   simulator->init();
   simulatorEuler->init();

   addObject(terrain);
   addObject(drapeau);
   addObject(drapeau2);

   createGlut(&argc, argv);

   glutIdleFunc(idle);
   timer.reset();
   glutMainLoop();

   delete terrain;
   delete drapeau;
   delete drapeau2;
   delete simulator;

   return 0;
}

