/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/
#include <GL/glut.h>
#include <iostream>

#include "deltaplane.h"
#include "terrain.h"
#include "timer.h"


void WriteImage();

//#define TRAINE

float        distanceCamera;
float        angleSol;
float        angleRotation;
float        refX, refY, refZ;

int          mode;
int          curX, curY;

std::vector<const Object*>   objects;
int  createGlut(int* argc, char** argv);

void addObject(const Object* object);


/*class Terrain : public Object{
  public:
    void renderGL() const{
        glPushMatrix();

        for(int x=-10;x<10;x++)
           for(int y=-10;y<10;y++){
             if((x+y) % 2 == 0)
                 glColor3ub(22, 150, 1);
               else
                 glColor3ub(32, 185, 4);
             glNormal3f(0, 0, 1);
             glBegin(GL_POLYGON);
               glVertex3d( 10*(x+1), 10*y, -0.05);
               glVertex3d( 10*(x+1), 10*(y+1), -0.05);
               glVertex3d( 10*x, 10*(y+1), -0.05);
               glVertex3d( 10*x, 10*y, -0.05);
             glEnd();
           }
        glPopMatrix();
    }
};*/

Timer                     timer;
Terrain*                  terrain;
DeltaPlane*               deltaplane;
DeltaPlane*               deltaplane2;
PhysicSimulatorVerlet*    simulator;

long          step;


#if defined(TRAINE)
Point*                    traine;
int                       bTraine, eTraine;
int                       nbTraine;
#endif

void idle(){
   int n = timer.getNbStepToDo();
   if(n>0){
      for(int i=0;i<n;i++){
         simulator->iter();
         //deltaplane->stabilize();
         Vector velocity = simulator->getVelocity(deltaplane->getPerson());
         //std::cout << step << " " << velocity.x << " " << velocity.y << " " << velocity.z << std::endl;
         Point p = deltaplane->getPosPerson();
#if defined(TRAINE)
         traine[eTraine++] = p;
         if(eTraine>=nbTraine) eTraine = 0;
         if(eTraine==bTraine) bTraine = (eTraine+1) % nbTraine;
#endif
         refX = -p.x;
         refY = -p.y;
         refZ = -p.z;
         step++;
      }
      Vector velocity = simulator->getVelocity(deltaplane->getPerson());
      //std::cout << "Step: " << step << " Velocity=" << velocity << "  Speed=" << velocity.norm() << " m/s" << std::endl;
      glutPostRedisplay();
   }
}

void mySpecialKey(int key, int x, int y){
   double dx=0, dy=0;
   switch(key){
      case GLUT_KEY_LEFT:
        dx += 0.01;
        break;
      case GLUT_KEY_RIGHT:
        dx -= 0.01;
        break;
      case GLUT_KEY_UP:
        dy -= 0.05;
        break;
      case GLUT_KEY_DOWN:
        dy += 0.05;
        break;
      case GLUT_KEY_F1:
        deltaplane->moveTo(Point(0, 150, 135));
        break;
      default:
        std::cout << "special key: " << key << std::endl;
        break;
   }

   deltaplane->controlPosition(dx, dy);
   //deltaplane2->controlPosition(dx, dy);
}

int main(int argc, char** argv){
#if defined(TRAINE)
   nbTraine = 1000;
   traine = new Point[nbTraine];
   bTraine = eTraine = 0;
#endif

   //terrain = Terrain::generateSinSin(100, 100, 6);//new Terrain();
   //terrain = new Terrain(100, 100, -50, -50, 2, 1);
   //terrain->computeVNs();
   //terrain = Terrain::loadFromJPEG("/home/eric/tmp/33/terrain.jpg", 1, 0.1);
   terrain = Terrain::loadFromJPEG("terrain.jpg", 1, 0.1);
   if(!terrain)
	   terrain = Terrain::generateSinSin(100, 100, 6);//new Terrain();
   //deltaplane = new DeltaPlane(Point(0, 150, 135), 3.5, 1.75, 6, 10.0, 20.0);
   //deltaplane = new DeltaPlane(Point(0, 150, 135), 3.5, 2.0, 8, 15.0, 45.0);
   deltaplane = new DeltaPlane(Point(0, 150, 135), 5.5, 3.0, 8, 15.0, 45.0);
   //deltaplane = new DeltaPlane(Point(0, 150, 135), 3.5, 1.75, 6, 15.0, 40.0);
   //deltaplane2 = new DeltaPlane(Point(0, 140, 135), 3.5, 1.75, 6, 20.0, 45.0);
   simulator = new PhysicSimulatorVerlet();
   //simulator->setWind(Vector(0, -4, 2));
   simulator->setWind(Vector(0, 0, 0));
   simulator->setTerrain(terrain);

   deltaplane->registerInSimulator(simulator);
   //deltaplane2->registerInSimulator(simulator);

   addObject(terrain);
   addObject(deltaplane);
   //addObject(deltaplane2);

   createGlut(&argc, argv);
   //glClearColor(1, 1, 1, 0);
   glClearColor(0.3, 0.4, 0.85, 0);

   glutSpecialFunc(mySpecialKey);
   glutIdleFunc(idle);
   step = 0;
   simulator->init();
   timer = Timer(100, 4);
   //timer = Timer(100, 1);
   timer.reset();
   glutMainLoop();

   delete terrain;
   delete deltaplane;
   //delete drapeau2;
   delete simulator;

   return 0;
}

void addObject(const Object* o){
   objects.push_back(o);
}

void init(){
    GLfloat pos[4] = {-400, 400.0, 400.0, 1.0 };
    glLightfv( GL_LIGHT0, GL_POSITION, pos );
    //glEnable( GL_CULL_FACE );
    glEnable( GL_LIGHTING );
    glEnable( GL_LIGHT0 );
    glEnable( GL_NORMALIZE );
    glEnable(GL_COLOR_MATERIAL);
    glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,GL_TRUE);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_ALPHA_TEST);
    glShadeModel(GL_SMOOTH);

    distanceCamera = 10;
    angleSol = 30;
    angleRotation = -170;
    refX = refY = 0;
    refZ = -2;
    mode = 0;
}

void display(){
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glTranslatef(0, 0, - distanceCamera);
    glRotatef(angleSol-90, 1, 0, 0);
    glRotatef(angleRotation, 0,0,1);
    glTranslatef(refX, refY, refZ);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

#if defined(TRAINE)
    glColor3ub(255, 149, 11);
    glBegin(GL_LINE_STRIP);
    for(int i=bTraine;i!=eTraine;i=(i+1)%nbTraine){
       glVertex3d(traine[i].x, traine[i].y, traine[i].z);
    }
    glEnd();
#endif

    for(unsigned int i=0;i<objects.size();i++)
        objects[i]->renderGL();

    //simulator->renderGL();

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();

    glColor3f(1, 0, 0);
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    Vector velocity = simulator->getVelocity(deltaplane->getPerson());
    Point pos = deltaplane->getPosPerson();
    char buf[256];
    double Z = terrain->getLevel(pos.x, pos.y);
    sprintf(buf, "Time=%.02lf Terrain=%0.1lf Pos=(%0.1lf, %0.1lf, %0.1lf) Vel=(%0.1lf, %0.1lf, %0.1lf)  Speed=%0.1lf", step/100.0, Z, pos.x, pos.y, pos.z, velocity.x, velocity.y, velocity.z, velocity.norm());
    glRasterPos3d(-5.1, -3.8, -10);
	for (int i = 0; buf[i]; i++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, buf[i]);
    glEnable( GL_LIGHTING );
    glEnable(GL_DEPTH_TEST);

    glFlush();
    //if(step%4 == 0)
    //    WriteImage();
    glutSwapBuffers();
}

void reshape(int width, int height){
    GLfloat w = (float) width / (float) height;
    GLfloat h = 1.0;

    glViewport( 0, 0, width, height );
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum( -w, w, -h, h, 2.5, 500.0 );
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void mousefonc(int button, int state, int x, int y){
   //GLUT_LEFT_BUTTON
   //GLUT_UP, GLUT_DOWN
   if(button==GLUT_LEFT_BUTTON && state==GLUT_DOWN){
      mode = (glutGetModifiers() & GLUT_ACTIVE_SHIFT)==0 ? 1 : 2;
      curX = x;
      curY = y;
      glutGetModifiers();
   }
   else
      mode = 0;
}

void mousemove(int x, int y){
   int dx = curX - x;
   int dy = curY - y;
   switch(mode){
      case 2:
         distanceCamera += dy / 10.0;
         if(distanceCamera<2) distanceCamera = 2;
         else if(distanceCamera>1000) distanceCamera = 1000;
         glutPostRedisplay();
         break;
      case 1:
         angleSol += dy * 0.25;
         if(angleSol < 0) angleSol = 0;
         else if(angleSol > 90) angleSol =90;
         angleRotation += -dx * 0.25;
         glutPostRedisplay();
         break;
   }
   curX = x;
   curY = y;
}

void special(int key, int x, int y){
   std::cout << "special key: " << key << std::endl;
}

void keyboard(unsigned char key, int x, int y){
   std::cout << "key: " << (int) key << std::endl;
   if(key == 27) exit(0);
}

int createGlut(int* argc, char** argv){
   glutInit(argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
   glutInitWindowSize(640, 480);
   glutInitWindowPosition(100,100);
   glutCreateWindow(argv[0]);
   init();
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutMouseFunc(mousefonc);
   glutMotionFunc(mousemove);
   glutKeyboardFunc(keyboard);
   //glutSpecialFunc(special);
   return 0;
}

void WriteImage()
 {
  static int cptImages=0;
  GLsizei gl_w=640;
  GLsizei gl_h=480;
  static char nomFichier[256];
  static char bits[800*800*3+200];

  glFinish();
  glPixelStorei( GL_PACK_ALIGNMENT, 4);
  glPixelStorei( GL_PACK_ROW_LENGTH, 0 );
  glPixelStorei( GL_PACK_SKIP_PIXELS, 0 );
  glPixelStorei( GL_PACK_SKIP_ROWS, 0 );

  glReadPixels(0,0,gl_w,gl_h, GL_RGB, GL_UNSIGNED_BYTE,
        bits);
  if (cptImages < 2000)
          {
                sprintf(nomFichier,"img%04d.ppm",cptImages++);

          FILE * fichier = fopen(nomFichier,"w");
          fprintf(fichier,"P6\n#Image\n%d %d\n255\n",gl_w,gl_h);

          int i, m=gl_w*3;
          for (i=gl_h-1; i>=0; i--) {
            fwrite( &(bits[i*m]), sizeof(unsigned char), m, fichier);
                }
          fclose(fichier);
          }
}

